/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

function WebAPI(nav, ui, settings) {
	this.nav = nav;
	this.ui = ui;
	this.settings = settings;
	
	this.searching = false;
	this.route_status = '';
	this.t_prev = $.now() - 1;
}


// Route
WebAPI.prototype.set_route = function(lat1, lng1, lat2, lng2) {
	if (this.searching)
		return;
	
	if (this.t_prev < $.now()) { // Avoid several calls in a small time
		this.t_prev = $.now() + 9000;
		
		this.searching = true;
		this.route_status = this.nav.get_route_status();
		
		// API parameters
		var avoid_tolls = '';
		switch (this.settings.get_routing_mode()) {
			case 2:
				var routing_mode = ',"costing":"bicycle"';
				break;
			case 1:
				var routing_mode = ',"costing":"pedestrian"';
				break;
			default:
				var routing_mode = ',"costing":"auto"';
				if (this.settings.get_avoid_tolls())
					avoid_tolls = ',"costing_options":{"auto":{"toll_booth_penalty":100000.0}}';
		}
		
		// API https://mapzen.com/projects/valhalla
		$.ajax({
			url: 'https://valhalla.mapzen.com/route',
			data: {
				json: '{"locations":[{"lat":'+lat1+',"lon":'+lng1+'},{"lat":'+lat2+',"lon":'+lng2+'}]'+routing_mode+avoid_tolls+'}',
				api_key: 'valhalla-4DcFzeQ' // TODO debug
			},
			dataType: 'json',
			timeout: 90000,
			success: this.OK_callback_set_route.bind(this),
			error: this.KO_callback_set_route.bind()
		});
		
	}
}

WebAPI.prototype.OK_callback_set_route = function(data) {
	this.searching = false;
	
	if (data.trip.status == 0) {
		this.nav.set_route(	(data.trip.summary.length*1000),
							data.trip.summary.time, // totals (meters & seconds)
							data.trip.legs[0].shape, // Map line
							data.trip.legs[0].maneuvers // Route stracks
		);
	}
	else {
		if (this.route_status == 'calc_from_out')
			this.nav.set_route_status('calc_from_out'); // recalculate
		else
			this.nav.set_route_status('errorAPI');
	}
}

WebAPI.prototype.KO_callback_set_route = function() {
	this.searching = false;
	
	if (this.route_status == 'calc_from_out')
		this.nav.set_route_status('calc_from_out'); // recalculate
	else
		this.nav.set_route_status('errorAPI');
}


// Simulate
WebAPI.prototype.simulate = function(lat1, lng1, lat2, lng2) {
	// API parameters
	var avoid_tolls = '';
	switch (this.settings.get_routing_mode()) {
		case 2:
			var routing_mode = ',"costing":"bicycle"';
			break;
		case 1:
			var routing_mode = ',"costing":"pedestrian"';
			break;
		default:
			var routing_mode = ',"costing":"auto"';
			if (this.settings.get_avoid_tolls())
				avoid_tolls = ',"costing_options":{"auto":{"toll_booth_penalty":100000.0}}';
	}
	
	// API https://mapzen.com/projects/valhalla
	$.ajax({
		url: 'https://valhalla.mapzen.com/route',
		data: {
			json: '{"locations":[{"lat":'+lat1+',"lon":'+lng1+'},{"lat":'+lat2+',"lon":'+lng2+'}]'+routing_mode+avoid_tolls+'}',
			api_key: 'valhalla-4DcFzeQ' // TODO debug
		},
		dataType: 'json',
		timeout: 90000,
		success: this.OK_callback_simulate.bind(this),
		error: this.KO_callback_simulate.bind()
	});
	
}

WebAPI.prototype.OK_callback_simulate = function(data) {
	if (data.trip.status == 0) {
		this.nav.set_route(	(data.trip.summary.length*1000),
							data.trip.summary.time,     // totals (meters & seconds)
							data.trip.legs[0].shape,    // Map line
							data.trip.legs[0].maneuvers // Route stracks
		);
		// Do everything here for not waiting GPS signals
		this.nav.set_route_status('simulate_drawing');
		this.ui.markers_POI_clear();
		this.ui.route(true);
		switch (this.settings.get_routing_mode()) {
			case 2:
				this.nav.set_route_status('simulate_done_bike');
				break;
			case 1:
				this.nav.set_route_status('simulate_done_walk');
				break;
			default:
				this.nav.set_route_status('simulate_done_car');
		}
	}
	else {
		this.nav.set_route_status('simulate_error');
	}
	this.ui.update();
}

WebAPI.prototype.KO_callback_simulate = function() {
	this.nav.set_route_status('simulate_error');
	this.ui.update();
}


// Radars
WebAPI.prototype.set_radars = function(lat1, lng1, lat2, lng2) {
	// Refresh current radars
	this.nav.radars_clear();
	this.ui.markers_radar_clear();
	
	// Search radars?
	if (!this.settings.get_alert_radars() || this.settings.get_routing_mode() != 0)
		return;
	
	// Search in extra border 5km
	var bbox = "[bbox=" +
			   (Number(Math.min(lng1, lng2)) - 0.049999989569187164 + ",").toString() +
			   (Number(Math.min(lat1, lat2)) - 0.049999989569187164 + ",").toString() +
			   (Number(Math.max(lng1, lng2)) + 0.049999989569187164 + ",").toString() +
			   (Number(Math.max(lat1, lat2)) + 0.049999989569187164).toString() + "]";
	
	// Search radars POI http://wiki.openstreetmap.org/wiki/Overpass_API
	$.ajax({
		url: 'https://www.overpass-api.de/api/xapi?node[highway=speed_camera]' + bbox,
		timeout: 90000,
		dataType: 'xml',
		success: this.OK_callback_set_radars.bind(this),
		error: this.KO_callback_set_radars.bind()
	});
}

WebAPI.prototype.OK_callback_set_radars = function(xml) {
	$('#p_txt_ind').html(t("Analyzing speed cameras… This could take a while"));
	setTimeout(this.analize_radars, 75, xml); // Need this setTimeout because the message will not apply if not
}

WebAPI.prototype.analize_radars = function(xml) {
	// For each radar...
	var aux_nav = this.nav;
	$(xml).find('node').each(function() {
		var xml_lat = parseFloat($(this).attr("lat"));
		var xml_lng = parseFloat($(this).attr("lon"));
		var xml_maxspeed = '!';
		$(this).find('tag').each(function(){
			if ($(this).attr("k") == 'maxspeed')
				xml_maxspeed = $(this).attr("v");
		});
		// Set radars for navigation
		aux_nav.set_radar(xml_lat, xml_lng, xml_maxspeed);
	});
	//// Show them
	this.ui.markers_radar_set(this.nav.get_radars());
}

WebAPI.prototype.KO_callback_set_radars = function() {
	this.ui.KO_API_radars();
}
