/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import "components"

Page {
    id: routePage

    header: PageHeader {
        title: i18n.tr("Route")
        flickable: flickable
    }

    Component.onCompleted: mainPageStack.executeJavaScript("qml_set_route_status();")

    Flickable {
        id: flickable

        anchors { fill: parent; margins: units.gu(2); topMargin: units.gu(4) }
        height: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > routePage.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        ListModel {
            id: routePageModel
            Component.onCompleted: initialize()

            function initialize() {
                routePageModel.append({mode: "SEARCH", text: i18n.tr("Search"), source: "../nav/img/pages/route/search.svg", visible: true})
                routePageModel.append({mode: "FAVORITES", text: i18n.tr("Favorites"), source: "../nav/img/pages/route/favorites.svg", visible: true})
                routePageModel.append({mode: "NEARBY", text: i18n.tr("Nearby"), source: "../nav/img/pages/route/nearby.svg", visible: true})
                routePageModel.append({mode: "COORDINATES", text: i18n.tr("Coordinates"), source: "../nav/img/pages/route/coordinates.svg", visible: true})
                routePageModel.append({mode: "SHARE", text: i18n.tr("Share"), source: "../nav/img/pages/route/share.svg", visible: true})
                routePageModel.append({mode: "CANCEL", text: i18n.tr("Cancel Route"), source: "../nav/img/pages/route/cancel.svg", visible: mainPageStack.routeState !== 'no'})
            }
        }

        CustomGridView {
            id: routePageGrid

            model: routePageModel
            delegate: GridDelegate {
                id: delegate

                title: model.text
                icon: model.source
                visible: model.visible

                onClicked: {
                    mainPageStack.pop(routePage)
                    if (model.mode === "FAVORITES") {
                        mainPageStack.push(Qt.resolvedUrl("FavoritesPage.qml"))
                    } else if (model.mode === "SEARCH") {
                        mainPageStack.push(Qt.resolvedUrl("SearchPage.qml"))
                    } else if (model.mode === "NEARBY") {
                        mainPageStack.push(Qt.resolvedUrl("Nearby.qml"))
                    } else if (model.mode === "COORDINATES") {
                        mainPageStack.push(Qt.resolvedUrl("Coordinate.qml"))
                    } else if (model.mode === "SHARE") {
                        mainPageStack.push(Qt.resolvedUrl("SharePage.qml"))
                    } else if (model.mode === "CANCEL") {
                        mainPageStack.routeState = 'no';
                        mainPageStack.executeJavaScript("click_cancel_route();")
                    }
                }
            }
        }
    }
}

