/*
 * uNav http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan https://launchpad.net/~nik90
 *
 * uNav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * uNav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import "components"

Page {
    id: sharePage

    header: PageHeader {
        title: i18n.tr("Share Position")
        flickable: flickable
        
        leadingActionBar.actions: Action {
            iconName: "back"
            text: i18n.tr("Back")
            shortcut: "Escape"
            onTriggered: {
                mainPageStack.pop(sharePage)
                mainPageStack.push(Qt.resolvedUrl("RoutePage.qml"))
            }
        }
    }

    Flickable {
        id: flickable

        anchors { fill: parent; topMargin: units.gu(4); margins: units.gu(2) }
        height: contentItem.childrenRect.height
        boundsBehavior: (contentHeight > sharePage.height) ? Flickable.DragAndOvershootBounds : Flickable.StopAtBounds

        ListModel {
            id: sharePageModel

            Component.onCompleted: initialize()

            function initialize() {
                sharePageModel.append({mode: "CURRENT", text: i18n.tr("Current"), source: "../nav/img/pages/share/current.svg", visible: (mainPageStack.center_onpos !== 0 && mainPageStack.currentLat !== "null" && mainPageStack.currentLng !== "null") })
                sharePageModel.append({mode: "DESTINATION", text: i18n.tr("Destination"), source: "../nav/img/pages/share/destination.svg", visible: (mainPageStack.routeState !== 'no' && mainPageStack.endLat !== "null" && mainPageStack.endLng !== "null") })
                sharePageModel.append({mode: "MAP", text: i18n.tr("From Map"), source: "../nav/img/pages/share/fromMap.svg", visible: true })
            }
        }

        CustomGridView {
            id: sharePageGrid

            model: sharePageModel
            delegate: GridDelegate {
                id: delegate

                title: model.text
                icon: model.source
                visible: model.visible

                onClicked: {
                    if (model.mode === "CURRENT") {
                        PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": mainPageStack.currentLat, "lon": mainPageStack.currentLng})
                    } else if (model.model === "DESTINATION") {
                        PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": mainPageStack.endLat, "lon": mainPageStack.endLng})
                    } else if (model.mode === "MAP") {
                        mainPageStack.pop(sharePage); // Show map
                    }
                }
            }
        }
    }
}
